"""
 eez_scpi_send.py

 EEZ PSU SCPI-based remote control speed tester
 Use it with Firmware M3 or newer

 Copyright (C) 2016 Envox d.o.o.

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""

import sys
import serial
import re
from serial.tools import list_ports
from time import sleep

U_MIN = 0.1
U_MAX = 5.1
U_STEP = 5
I_MAX = 2

def result():
    result = ''
    while True:
        chars = ser.read()
        if len(chars) == 0:
            break
        for c in chars:
            result += chr(c)
    if result != '':
        print(result)
    return result

def command(cmd, readResult=False):
    print(cmd)
    ser.write((cmd + "\n").encode())
    if readResult:
        return result()

def probeSerialPort(port):
    try:
        global ser
        ser = serial.Serial(port, 9600, timeout=1)
        for i in range(3):
            if 'EEZ' in command('*idn?', True):
                return True
    except serial.SerialException:
        pass
    return False

def runCommands():
    u = U_MIN
    command('outp:trac on') # Set tracking mode to control both channels simultaneously
    command('volt:lim max') # Set voltage limit to max
    command('curr:lim max') # Set current limit to max
    command('curr ' + ("%.2f" % I_MAX)) # Set output current
    command('outp on') # Enable output

    while True: 
        command('volt ' + ("%.2f" % u))

        sleep(0.04167) # delay, e.g. set 0.04167 for 12 Hz

        u += U_STEP
        if u > U_MAX: u = U_MIN

if __name__ == "__main__":
    if len(sys.argv) == 2:
        if probeSerialPort(sys.argv[1]):
            runCommands()
        else:
            print("EEZ PSU not found on port '" + sys.argv[1] + "'")
    else:
        for port in list_ports.comports():
            print("Probing serial port '" + port[0] + "'...")
            if probeSerialPort(port[0]):
                print()
                print("EEZ PSU found on port '" + port[0] + "'")
                print()
                runCommands()
                break
        else:
            print("EEZ PSU not found")
